import 'package:diagram_editor_apps/simple_diagram_editor/data/node_data/base_node_data.dart';
import 'package:diagram_editor_apps/simple_diagram_editor/data/workflow_node_connector.dart';
import 'package:diagram_editor_apps/simple_diagram_editor/data/worklow_node.dart';
import 'package:flutter/material.dart';
import 'package:collection/collection.dart';

class NodeData {
  final String title;
  final String description;
  final IconData iconData;
  final WorkflowNodeType type;
  final List<WorkflowNodeConnectorTrigger> possibleTriggers;
  List<WorkflowNodeConnector> connectors;
  BaseNodeData? data;
  bool isHighlightVisible = false;

  NodeData({
    required this.title,
    required this.description,
    required this.type,
    required this.iconData,
    required this.connectors,
    required this.possibleTriggers,
    this.data,
  });

  switchHighlight() {
    isHighlightVisible = !isHighlightVisible;
  }

  showHighlight() {
    isHighlightVisible = true;
  }

  hideHighlight() {
    isHighlightVisible = false;
  }

  bool hasLinkedTrigger(WorkflowNodeConnectorTrigger trigger, String sourceId) {
    if (connectors.isEmpty) return false;
    var hasTrigger = connectors.any((element) => element.trigger != null);
    if (hasTrigger) {
      var connector = connectors.firstWhereOrNull((element) =>
          element.trigger! == trigger && element.fromNodeId == sourceId);
      if (connector != null) {
        return true;
      }
    }
    return false;
  }
}

extension NodeTypeValue on WorkflowNodeType {
  String get value {
    switch (this) {
      case WorkflowNodeType.Start:
        return "start";
      case WorkflowNodeType.Stop:
        return "stop";
    }

    return "";
  }

  String get linkTitle {
    switch (this) {
      case WorkflowNodeType.Start:
        return "";
      case WorkflowNodeType.Stop:
        return "End flow";
    }

    return "";
  }
}
