import 'package:flutter/material.dart';

enum XInputType { text, password, email, multiline }
enum XInputSize { small, large }

class XInput extends StatelessWidget {
  final String labelName;
  final bool isRequired;
  final XInputSize inputSize;
  final String? infoLabel;
  final bool autoFocus;
  final bool obscureText;
  final bool readOnly;
  final Icon? icon;
  final Function(String value)? onSubmit;

  const XInput(
      {Key? key,
      required this.labelName,
      this.infoLabel,
      this.autoFocus = false,
      this.readOnly = false,
      this.icon,
      this.isRequired = false,
      this.inputSize = XInputSize.large,
      this.obscureText = false,
      this.onSubmit})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Flexible(
      child: Padding(
        padding: const EdgeInsets.only(top: 10, bottom: 10),
        child: textInputFormField,
      ),
    );
  }

  Widget get textInputFormField {
    return TextFormField(
        style: TextStyle(fontSize: 12, fontWeight: FontWeight.bold),
        obscureText: false,
        textInputAction: TextInputAction.done,
        onFieldSubmitted: onSubmit,
        decoration: InputDecoration(
          border: const OutlineInputBorder(
            borderSide: BorderSide(color: Color(0xFFEFF0F1), width: 1.0),
          ),
          enabledBorder: const OutlineInputBorder(
            borderSide: BorderSide(color: Color(0xFFEFF0F1), width: 1.0),
          ),
          focusedBorder: const OutlineInputBorder(
            borderSide: BorderSide(color: Color(0xFFEFF0F1), width: 1.0),
          ),
          errorBorder: OutlineInputBorder(
            borderSide:
                BorderSide(color: Colors.red[400] ?? Colors.red, width: 1.0),
          ),
          suffixIcon: icon,
          labelText: labelName,
          contentPadding: const EdgeInsets.fromLTRB(20, 10, 10, 10),
        ));
  }

  get textFieldSize {
    switch (inputSize) {
      case XInputSize.small:
        return 150.0;
      case XInputSize.large:
        return 400.0;
    }
  }
}
