import 'package:diagram_editor/diagram_editor.dart';
import 'package:diagram_editor_apps/simple_diagram_editor/data/building_blocks.dart';
import 'package:diagram_editor_apps/simple_diagram_editor/data/worklow_node.dart';
import 'package:diagram_editor_apps/simple_diagram_editor/policy/my_policy_set.dart';
import 'package:flutter/material.dart';
import 'package:diagram_editor_apps/simple_diagram_editor/data/node_data.dart';

class DraggableMenu extends StatelessWidget {
  final MyPolicySet myPolicySet;

  const DraggableMenu({
    Key? key,
    required this.myPolicySet,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return ListView(
      children: <Widget>[
        ...buildingBlocks.map(
          (componentType) {
            var componentData = getComponentData(componentType.type.value);
            return Padding(
              padding: EdgeInsets.fromLTRB(8, 8, 8, 8),
              child: LayoutBuilder(
                builder: (context, constraints) {
                  return SizedBox(
                    width: constraints.maxWidth < componentData.size.width
                        ? componentData.size.width *
                            (constraints.maxWidth / componentData.size.width)
                        : componentData.size.width,
                    height: constraints.maxWidth < componentData.size.width
                        ? componentData.size.height *
                            (constraints.maxWidth / componentData.size.width)
                        : componentData.size.height,
                    child: Align(
                      alignment: Alignment.center,
                      child: AspectRatio(
                        aspectRatio: componentData.size.aspectRatio,
                        child: Tooltip(
                          message: componentData.type ?? "",
                          child: DraggableComponent(
                            myPolicySet: myPolicySet,
                            componentData: componentData,
                            menu: true,
                          ),
                        ),
                      ),
                    ),
                  );
                },
              ),
            );
          },
        ).toList(),
      ],
    );
  }

  ComponentData getComponentData(String componentType, {String? id = null}) {
    if (componentType == WorkflowNodeType.Stop.value) {
      return ComponentData(
        id: id,
        minSize: Size(10, 5),
        size: Size(50, 50),
        data: buildingBlocks
            .firstWhere((element) => element.type.value == componentType),
        type: componentType,
      );
    }

    return ComponentData(
      id: id,
      minSize: Size(10, 5),
      size: Size(350, 250),
      data: buildingBlocks
          .firstWhere((element) => element.type.value == componentType),
      type: componentType,
    );
  }
}

class DraggableComponent extends StatelessWidget {
  final MyPolicySet myPolicySet;
  final ComponentData componentData;
  final bool menu;

  const DraggableComponent({
    Key? key,
    required this.myPolicySet,
    required this.componentData,
    required this.menu,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Draggable<ComponentData>(
      affinity: Axis.horizontal,
      ignoringFeedbackSemantics: true,
      data: componentData,
      childWhenDragging: myPolicySet.showComponentBody(componentData),
      feedback: Material(
        color: Colors.transparent,
        child: Container(
          width: componentData.size.width,
          height: componentData.size.height,
          child: myPolicySet.showComponentBody(componentData),
        ),
      ),
      child: myPolicySet.showComponentBody(componentData),
    );
  }
}
