import 'package:diagram_editor/diagram_editor.dart';
import 'package:diagram_editor_apps/simple_diagram_editor/data/node_data.dart';
import 'package:flutter/material.dart';
import 'package:popover/popover.dart';

class XBaseNode extends StatelessWidget {
  final bool isStop;
  final ComponentData componentData;
  final NodeData nodeData;
  final ComponentDesignPolicy policy;

  XBaseNode(
      {Key? key,
      required this.nodeData,
      required this.componentData,
      this.isStop = false,
      required this.policy})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Container(
      decoration: BoxDecoration(
          borderRadius: BorderRadius.all(
            Radius.circular(10),
          ),
          boxShadow: [
            BoxShadow(
              color: Colors.black,
              blurRadius: 2.0,
              spreadRadius: 0.0,
              offset: Offset(2.0, 2.0), // shadow direction: bottom right
            )
          ],
          color: Colors.white),
      child: isStop
          ? Container(
              width: 35,
              height: 35,
              decoration: BoxDecoration(
                  borderRadius: BorderRadius.all(
                    Radius.circular(10),
                  ),
                  color: Colors.green),
              child: Center(
                child: Icon(
                  nodeData.iconData,
                  color: Colors.white,
                ),
              ),
            )
          : Column(
              children: [
                Row(
                  children: [
                    Padding(
                      padding: const EdgeInsets.all(8.0),
                      child: Container(
                        width: 35,
                        height: 35,
                        decoration: BoxDecoration(
                            borderRadius: BorderRadius.all(
                              Radius.circular(10),
                            ),
                            color: Colors.green),
                        child: Center(
                          child: Icon(
                            nodeData.iconData,
                            color: Colors.white,
                          ),
                        ),
                      ),
                    ),
                    SizedBox(
                      width: 30,
                    ),
                    Text(
                      nodeData.title,
                      style: TextStyle(fontWeight: FontWeight.bold),
                    ),
                  ],
                ),
                Divider(),
                Padding(
                  padding: const EdgeInsets.all(8.0),
                  child: Text(nodeData.description),
                ),
                SizedBox(
                  height: 30,
                ),
                Container(
                  width: 80,
                  height: 40,
                  decoration: const BoxDecoration(
                    color: Colors.white,
                    borderRadius: BorderRadius.all(Radius.circular(5)),
                    boxShadow: [
                      BoxShadow(color: Colors.black26, blurRadius: 5)
                    ],
                  ),
                  child: GestureDetector(
                    child: const Center(child: Text('Click Me')),
                    onTap: () {
                      showPopover(
                        context: context,
                        transitionDuration: const Duration(milliseconds: 150),
                        bodyBuilder: (context) => ListItems(),
                        onPop: () => print('Popover was popped!'),
                        direction: PopoverDirection.right,
                        width: 300,
                        height: 500,
                      );
                    },
                  ),
                )
              ],
            ),
    );
  }
}

class ListItems extends StatelessWidget {
  const ListItems({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Scrollbar(
      child: Padding(
        padding: const EdgeInsets.symmetric(vertical: 8),
        child: ListView(
          padding: const EdgeInsets.all(8),
          children: [
            InkWell(
              onTap: () {},
              child: Container(
                height: 50,
                color: Colors.amber[100],
                child: const Center(child: Text('Entry A')),
              ),
            ),
            const Divider(),
            Container(
              height: 50,
              color: Colors.amber[200],
              child: const Center(child: Text('Entry B')),
            ),
            const Divider(),
            Container(
              height: 50,
              color: Colors.amber[300],
              child: const Center(child: Text('Entry C')),
            ),
          ],
        ),
      ),
    );
  }
}
